import { Observable, Subject } from "rxjs";

export type TransportEvent = 'Connection' | 'New Message'
export type TransportType = 'WEBSOCKET' | 'GRPC' | 'HTTP' | 'KAFKA' | undefined
export interface ITransport {
    getEventNotification(): Observable<TransportEventNotification>
}

export interface ITransportTransmitting extends ITransport {
    transmit(message: TransportMessage): void
}

export interface ITransportReceiving extends ITransport {
    receive(): Observable<any>
}

export interface ITransportSuper extends ITransportTransmitting, ITransportReceiving {
    // additional properties && methods  here
}
export interface TransportSettings {
    profileInfo: ProfileInfo,
}

export interface ProfileInfo {
    id: string,
    name: string,
    transport: TransportType,
    url?: string | null,
    port?: number | null
}

export interface TransmitterProfile {
    uuid: string,
    name: string,
    dateCreated: Date,
    transportType: TransportType,
    eventNotification: Subject<TransportEventNotification>,
    instance?: any
}

export interface ReceiverProfile {
    uuid: string,
    name: string,
    dateCreated: Date,
    transportType: TransportType,
    eventNotification: Subject<TransportEventNotification>,
    instance?: any
}

export interface TransportEventNotification {
    event: TransportEvent,
    description: string,
    transportType: TransportType,
    data?: any
}


export interface TransportMessage {
    id: string,
    receiverID: string,
    payload: any,
    event: string
}