import { interval, map, Observable, Subject } from "rxjs"
import { v4 as uuidv4 } from 'uuid'
import { FisMessage, TransportManager } from "../transport/transport.manager"

// need to instantiate to grab both incoming and outgoing

class Application {
    service!: Blue
    notification = new Subject<any>()
    constructor(service: Blue) {
        this.service = service

        // this one will directly send over 
        // this.subscribeNotification().subscribe(notification => this.service.transportManager.send(notification))
        this.notify()
    }

    subscribeNotification(): Observable<FisMessage> {
        return this.notification.asObservable()
    }

    notify() {
        // logic here
        let output = interval(1000)
        output.pipe(map(
            value => {
                let message: FisMessage = {
                    header: {
                        messageID: uuidv4(),
                        messageName: `NotificationMessage`
                    },
                    data: `Notification Message ${value}`
                }
                return message
            }
        )).subscribe(this.notification)
    }

    /* EXTRA. Appplication acting as a receiver */
    processRequest(request: FisMessage): Observable<FisMessage> {
        return new Observable((response) => {
            // process responses. Should be called by Blue after receiving a request
        })
    }
    // This two only exist at this level. 
    send(message: FisMessage): Observable<FisMessage> {
        return new Observable((response) => {
            // logic here
        })
    }
}

/* Actually this class will acquire a high level information on the receiver as well.
Because it's the retransmission that instantiated in this layer.
So blue will work closely with the transport manager to acquire the necessary notifications with regards to the receiver's 
information and it's status, but it should not concern itself with the intricacies of how the messages will be transmitted over. */
class Blue {
    transportManager!: TransportManager
    receiverProfiles: any[] = []
    constructor(transportManager: TransportManager) {
        this.transportManager = transportManager

        this.processIncomingRequest()
    }

    processIncomingRequest(): void {
        // prcoess incoming request based on the receiver's profile relevant request
    }

}


const application = new Application(new Blue(new TransportManager(3000, 'http://localhost:3000')))

/* Transmitter acting as a receiver */
// logic here